import { world } from "@minecraft/server";

// Custom component để áp dụng sát thương khi tấn công và gọi function hammer_sound
const DefenderBattleAxeComponent = {
    onHitEntity({ hitEntity, source }) {
        let chance = 0.10;  // 10% cơ hội gây sát thương
        let extraDamage = 60;  // Sát thương thêm

        // Áp dụng sát thương nếu xác suất xảy ra
        if (Math.random() < chance) {
            hitEntity.applyDamage(extraDamage);
            console.log(`Đã gây ${extraDamage} sát thương lên thực thể.`);

            // Lấy vị trí của thực thể bị đánh trúng
            const position = hitEntity.location;
            const dimension = hitEntity.dimension;

            // Phát âm thanh tại vị trí của thực thể bị đánh trúng
            dimension.runCommandAsync(`playsound hit.bonus @a ${position.x} ${position.y} ${position.z} 4`).then(() => {
                console.log(`Executed hammer_sound at position ${position.x}, ${position.y}, ${position.z}`);
            }).catch(error => {
                console.error(`Failed to execute playsound at position ${position.x}, ${position.y}, ${position.z}:`, error);
            });
        }
    }
};

// Đăng ký custom component fv:big_sword
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:defender_battle_axe", DefenderBattleAxeComponent);
});

// Tạo component custom fv:defender_halberd
const DefenderHalberdComponent = {
    onHitEntity({ hitEntity }) {
        let chance = 0.10;  // Mặc định là 10% cơ hội
        let effectDuration = 2;  // Mặc định là 5 giây
        let effectAmplifier = 10;  // Cấp độ của hiệu ứng luôn là 10

        // Áp dụng hiệu ứng nếu xác suất xảy ra
        const effects = ["slowness", "weakness", "blindness"];

        if (Math.random() < chance) {
            effects.forEach(effect => {
                hitEntity.addEffect(effect, effectDuration * 20, {
                    amplifier: effectAmplifier,
                    showParticles: true
                });
            });

            // Lấy vị trí của thực thể bị đánh trúng
            const position = hitEntity.location;
            const dimension = hitEntity.dimension;

            // Phát âm thanh tại vị trí của thực thể bị đánh trúng
            dimension.runCommandAsync(`playsound random.anvil_land @a ${position.x} ${position.y} ${position.z} 4`).then(() => {
                console.log(`Executed hammer_sound at position ${position.x}, ${position.y}, ${position.z}`);
            }).catch(error => {
                console.error(`Failed to execute playsound at position ${position.x}, ${position.y}, ${position.z}:`, error);
            });
        }
    }
};


// Đăng ký custom component fv:defender_halberd
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:defender_halberd", DefenderHalberdComponent);
});

const DefenderSwordComponent = {
    onHitEntity({ hitEntity, source }) {
        let chance = 0.02;  // 10% cơ hội gây sát thương
        let extraDamage = 120;  // Sát thương thêm

        // Áp dụng sát thương nếu xác suất xảy ra
        if (Math.random() < chance) {
            hitEntity.applyDamage(extraDamage);
            console.log(`Đã gây ${extraDamage} sát thương lên thực thể.`);

            // Lấy vị trí của thực thể bị đánh trúng
            const position = hitEntity.location;
            const dimension = hitEntity.dimension;

            // Phát âm thanh tại vị trí của thực thể bị đánh trúng
            dimension.runCommandAsync(`playsound item.trident.thunder @a ${position.x} ${position.y} ${position.z} 4`).then(() => {
                console.log(`Executed hammer_sound at position ${position.x}, ${position.y}, ${position.z}`);
            }).catch(error => {
                console.error(`Failed to execute playsound at position ${position.x}, ${position.y}, ${position.z}:`, error);
            });
        }
    }
};

// Đăng ký custom component fv:big_sword
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:defender_sword", DefenderSwordComponent);
});


