import { Entity, Player, world, EntityDamageCause } from "@minecraft/server";

const MeleeHornComponent = {
    onUse(event) {
        const player = event.source;
        if (!(player instanceof Player)) return;

        const position = player.location;
        const dimension = player.dimension;

        // Lấy item trong tay người chơi
        const itemStack = event.itemStack;
        if (!itemStack) return;

        // Kiểm tra và kích hoạt cooldown cho item
        const cooldownComponent = itemStack.getComponent("minecraft:cooldown");
        if (cooldownComponent && cooldownComponent.isValid()) {
            cooldownComponent.startCooldown(player);
        } else {
            console.warn("Cooldown component is not valid or not present on this item.");
            return; // Kết thúc nếu cooldown không hợp lệ
        }

        // Phát âm thanh tại vị trí của người chơi
        dimension.runCommandAsync(`playsound villager_call_horn @a ${position.x} ${position.y} ${position.z} 4`)
            .then(() => console.log("Played horn sound at player's position."))
            .catch(error => console.error("Failed to play horn sound:", error));

        // Kiểm tra tag của người chơi
        const tags = player.getTags();
        if (tags.includes("red") || tags.includes("blue") || tags.includes("grey")) {
            // Gây sát thương cho người chơi
            player.applyDamage(10, {
                cause: EntityDamageCause.magic
            });

            // Hiện dòng chữ trên thanh actionbar
            player.runCommandAsync(`title @s actionbar You are in another faction`)
                .then(() => console.log("Displayed actionbar message."))
                .catch(error => console.error("Failed to display actionbar message:", error));
            return; // Kết thúc nếu người chơi có tag không hợp lệ
        }

        // Tìm và dịch chuyển các thực thể trong phạm vi 30 block
        const entities = dimension.getEntities({
            location: position,
            radius: 30
        });

        entities.forEach(entity => {
            const familyComponent = entity.getComponent("minecraft:type_family");
            if (!familyComponent) return;

            // Kiểm tra thực thể có các family yêu cầu
            const hasRequiredFamilies = (
                familyComponent.hasTypeFamily("irongolem") &&
                (familyComponent.hasTypeFamily("swordman") || familyComponent.hasTypeFamily("axeman") || familyComponent.hasTypeFamily("spearman"))
            );

            if (hasRequiredFamilies) {
                entity.teleport(
                    position,
                    { dimension: dimension }
                );
            }
        });
    }
};

// Đăng ký custom component với tên fv:melee_horn
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:melee_horn", MeleeHornComponent);
});

const RangedHornComponent = {
    onUse(event) {
        const player = event.source;
        if (!(player instanceof Player)) return;

        const position = player.location;
        const dimension = player.dimension;

        // Lấy item trong tay người chơi
        const itemStack = event.itemStack;
        if (!itemStack) return;

        // Kiểm tra và kích hoạt cooldown cho item
        const cooldownComponent = itemStack.getComponent("minecraft:cooldown");
        if (cooldownComponent && cooldownComponent.isValid()) {
            cooldownComponent.startCooldown(player);
        } else {
            console.warn("Cooldown component is not valid or not present on this item.");
            return; // Kết thúc nếu cooldown không hợp lệ
        }

        // Phát âm thanh tại vị trí của người chơi
        dimension.runCommandAsync(`playsound villager_call_horn @a ${position.x} ${position.y} ${position.z} 4`)
            .then(() => console.log("Played horn sound at player's position."))
            .catch(error => console.error("Failed to play horn sound:", error));

        // Kiểm tra tag của người chơi
        const tags = player.getTags();
        if (tags.includes("red") || tags.includes("blue") || tags.includes("grey")) {
            // Gây sát thương cho người chơi
            player.applyDamage(10, {
                cause: EntityDamageCause.magic
            });

            // Hiện dòng chữ trên thanh actionbar
            player.runCommandAsync(`title @s actionbar You are in another faction`)
                .then(() => console.log("Displayed actionbar message."))
                .catch(error => console.error("Failed to display actionbar message:", error));
            return; // Kết thúc nếu người chơi có tag không hợp lệ
        }

        // Tìm và dịch chuyển các thực thể trong phạm vi 30 block
        const entities = dimension.getEntities({
            location: position,
            radius: 30
        });

        entities.forEach(entity => {
            const familyComponent = entity.getComponent("minecraft:type_family");
            if (!familyComponent) return;

            // Kiểm tra thực thể có các family yêu cầu
            const hasRequiredFamilies = (
                familyComponent.hasTypeFamily("irongolem") &&
                (familyComponent.hasTypeFamily("crossbowman") || familyComponent.hasTypeFamily("archer") || familyComponent.hasTypeFamily("gunner"))
            );

            if (hasRequiredFamilies) {
                entity.teleport(
                    position,
                    { dimension: dimension }
                );
            }
        });
    }
};

// Đăng ký custom component với tên fv:melee_horn
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:ranged_horn", RangedHornComponent);
});

const SpecialHornComponent = {
    onUse(event) {
        const player = event.source;
        if (!(player instanceof Player)) return;

        const position = player.location;
        const dimension = player.dimension;

        // Lấy item trong tay người chơi
        const itemStack = event.itemStack;
        if (!itemStack) return;

        // Kiểm tra và kích hoạt cooldown cho item
        const cooldownComponent = itemStack.getComponent("minecraft:cooldown");
        if (cooldownComponent && cooldownComponent.isValid()) {
            cooldownComponent.startCooldown(player);
        } else {
            console.warn("Cooldown component is not valid or not present on this item.");
            return; // Kết thúc nếu cooldown không hợp lệ
        }

        // Phát âm thanh tại vị trí của người chơi
        dimension.runCommandAsync(`playsound villager_call_horn @a ${position.x} ${position.y} ${position.z} 4`)
            .then(() => console.log("Played horn sound at player's position."))
            .catch(error => console.error("Failed to play horn sound:", error));

        // Kiểm tra tag của người chơi
        const tags = player.getTags();
        if (tags.includes("red") || tags.includes("blue") || tags.includes("grey")) {
            // Gây sát thương cho người chơi
            player.applyDamage(10, {
                cause: EntityDamageCause.magic
            });

            // Hiện dòng chữ trên thanh actionbar
            player.runCommandAsync(`title @s actionbar You are in another faction`)
                .then(() => console.log("Displayed actionbar message."))
                .catch(error => console.error("Failed to display actionbar message:", error));
            return; // Kết thúc nếu người chơi có tag không hợp lệ
        }

        // Tìm và dịch chuyển các thực thể trong phạm vi 30 block
        const entities = dimension.getEntities({
            location: position,
            radius: 30
        });

        entities.forEach(entity => {
            const familyComponent = entity.getComponent("minecraft:type_family");
            if (!familyComponent) return;

            // Kiểm tra thực thể có các family yêu cầu
            const hasRequiredFamilies = (
                familyComponent.hasTypeFamily("irongolem") &&
                (familyComponent.hasTypeFamily("special"))
            );

            if (hasRequiredFamilies) {
                entity.teleport(
                    position,
                    { dimension: dimension }
                );
            }
        });
    }
};

// Đăng ký custom component với tên fv:melee_horn
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:special_horn", SpecialHornComponent);
});

//BLUE

const BlueHornComponent = {
    onUse(event) {
        const player = event.source;
        if (!(player instanceof Player)) return;

        const position = player.location;
        const dimension = player.dimension;

        // Lấy item trong tay người chơi
        const itemStack = event.itemStack;
        if (!itemStack) return;

        // Kiểm tra và kích hoạt cooldown cho item
        const cooldownComponent = itemStack.getComponent("minecraft:cooldown");
        if (cooldownComponent && cooldownComponent.isValid()) {
            cooldownComponent.startCooldown(player);
        } else {
            console.warn("Cooldown component is not valid or not present on this item.");
            return; // Kết thúc nếu cooldown không hợp lệ
        }

        // Phát âm thanh tại vị trí của người chơi
        dimension.runCommandAsync(`playsound blue_horn @a ${position.x} ${position.y} ${position.z} 4`)
            .then(() => console.log("Played horn sound at player's position."))
            .catch(error => console.error("Failed to play horn sound:", error));

        // Kiểm tra tag của người chơi
        const tags = player.getTags();
        if (!tags.includes("blue")) {
            // Gây sát thương cho người chơi
            player.applyDamage(10, {
                cause: EntityDamageCause.magic
            });

            // Hiện dòng chữ trên thanh actionbar
            player.runCommandAsync(`title @s actionbar You are not blue member`)
                .then(() => console.log("Displayed actionbar message."))
                .catch(error => console.error("Failed to display actionbar message:", error));
            return; // Kết thúc nếu người chơi có tag không hợp lệ
        }

        // Tìm và dịch chuyển các thực thể trong phạm vi 30 block
        const entities = dimension.getEntities({
            location: position,
            radius: 30
        });

        entities.forEach(entity => {
            const familyComponent = entity.getComponent("minecraft:type_family");
            if (!familyComponent) return;

            // Kiểm tra thực thể có các family yêu cầu
            const hasRequiredFamilies = (
                familyComponent.hasTypeFamily("blue") &&
                (familyComponent.hasTypeFamily("melee") || familyComponent.hasTypeFamily("ranged"))
            );

            if (hasRequiredFamilies) {
                entity.teleport(
                    position,
                    { dimension: dimension }
                );
            }
        });
    }
};

// Đăng ký custom component với tên fv:melee_horn
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:blue_horn", BlueHornComponent);
});

//RED

const RedHornComponent = {
    onUse(event) {
        const player = event.source;
        if (!(player instanceof Player)) return;

        const position = player.location;
        const dimension = player.dimension;

        // Lấy item trong tay người chơi
        const itemStack = event.itemStack;
        if (!itemStack) return;

        // Kiểm tra và kích hoạt cooldown cho item
        const cooldownComponent = itemStack.getComponent("minecraft:cooldown");
        if (cooldownComponent && cooldownComponent.isValid()) {
            cooldownComponent.startCooldown(player);
        } else {
            console.warn("Cooldown component is not valid or not present on this item.");
            return; // Kết thúc nếu cooldown không hợp lệ
        }

        // Phát âm thanh tại vị trí của người chơi
        dimension.runCommandAsync(`playsound red_horn @a ${position.x} ${position.y} ${position.z} 4`)
            .then(() => console.log("Played horn sound at player's position."))
            .catch(error => console.error("Failed to play horn sound:", error));

        // Kiểm tra tag của người chơi
        const tags = player.getTags();
        if (!tags.includes("red")) {
            // Gây sát thương cho người chơi
            player.applyDamage(10, {
                cause: EntityDamageCause.magic
            });

            // Hiện dòng chữ trên thanh actionbar
            player.runCommandAsync(`title @s actionbar You are not red member`)
                .then(() => console.log("Displayed actionbar message."))
                .catch(error => console.error("Failed to display actionbar message:", error));
            return; // Kết thúc nếu người chơi có tag không hợp lệ
        }

        // Tìm và dịch chuyển các thực thể trong phạm vi 30 block
        const entities = dimension.getEntities({
            location: position,
            radius: 30
        });

        entities.forEach(entity => {
            const familyComponent = entity.getComponent("minecraft:type_family");
            if (!familyComponent) return;

            // Kiểm tra thực thể có các family yêu cầu
            const hasRequiredFamilies = (
                familyComponent.hasTypeFamily("red") &&
                (familyComponent.hasTypeFamily("melee") || familyComponent.hasTypeFamily("ranged"))
            );

            if (hasRequiredFamilies) {
                entity.teleport(
                    position,
                    { dimension: dimension }
                );
            }
        });
    }
};

// Đăng ký custom component với tên fv:melee_horn
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:red_horn", RedHornComponent);
});

//GREY

const GreyHornComponent = {
    onUse(event) {
        const player = event.source;
        if (!(player instanceof Player)) return;

        const position = player.location;
        const dimension = player.dimension;

        // Lấy item trong tay người chơi
        const itemStack = event.itemStack;
        if (!itemStack) return;

        // Kiểm tra và kích hoạt cooldown cho item
        const cooldownComponent = itemStack.getComponent("minecraft:cooldown");
        if (cooldownComponent && cooldownComponent.isValid()) {
            cooldownComponent.startCooldown(player);
        } else {
            console.warn("Cooldown component is not valid or not present on this item.");
            return; // Kết thúc nếu cooldown không hợp lệ
        }

        // Phát âm thanh tại vị trí của người chơi
        dimension.runCommandAsync(`playsound grey_horn @a ${position.x} ${position.y} ${position.z} 4`)
            .then(() => console.log("Played horn sound at player's position."))
            .catch(error => console.error("Failed to play horn sound:", error));

        // Kiểm tra tag của người chơi
        const tags = player.getTags();
        if (!tags.includes("grey")) {
            // Gây sát thương cho người chơi
            player.applyDamage(10, {
                cause: EntityDamageCause.magic
            });

            // Hiện dòng chữ trên thanh actionbar
            player.runCommandAsync(`title @s actionbar You are not grey member`)
                .then(() => console.log("Displayed actionbar message."))
                .catch(error => console.error("Failed to display actionbar message:", error));
            return; // Kết thúc nếu người chơi có tag không hợp lệ
        }

        // Tìm và dịch chuyển các thực thể trong phạm vi 30 block
        const entities = dimension.getEntities({
            location: position,
            radius: 30
        });

        entities.forEach(entity => {
            const familyComponent = entity.getComponent("minecraft:type_family");
            if (!familyComponent) return;

            // Kiểm tra thực thể có các family yêu cầu
            const hasRequiredFamilies = (
                familyComponent.hasTypeFamily("grey") &&
                (familyComponent.hasTypeFamily("melee") || familyComponent.hasTypeFamily("ranged"))
            );

            if (hasRequiredFamilies) {
                entity.teleport(
                    position,
                    { dimension: dimension }
                );
            }
        });
    }
};

// Đăng ký custom component với tên fv:melee_horn
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:grey_horn", GreyHornComponent);
});