import { world, Entity } from '@minecraft/server';

const AbsorptionComponent = {
    onCompleteUse(event) {
        const player = event.source; // Lấy người chơi (source entity)
        const itemStack = event.itemStack; // Lấy item stack hoàn thành việc sử dụng

        if (player instanceof Entity && itemStack) {
            // Áp dụng hiệu ứng strength cấp 0 trong 30 giây
            player.addEffect("absorption", 20 * 20, {
                amplifier: 1,  // Cấp độ 0
                showParticles: true
            });

            console.log("Applied speed to player.");
        } else {
            console.warn("Source is not a valid player or itemStack is missing.");
        }
    }
};

// Đăng ký custom component fv:spear_use_lv2
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:absorption_use", AbsorptionComponent);
});

const ResistanceComponent = {
    onCompleteUse(event) {
        const player = event.source; // Lấy người chơi (source entity)
        const itemStack = event.itemStack; // Lấy item stack hoàn thành việc sử dụng

        if (player instanceof Entity && itemStack) {
            // Áp dụng hiệu ứng strength cấp 0 trong 30 giây
            player.addEffect("resistance", 20 * 20, {
                amplifier: 1,  // Cấp độ 0
                showParticles: true
            });

            console.log("Applied speed to player.");
        } else {
            console.warn("Source is not a valid player or itemStack is missing.");
        }
    }
};

// Đăng ký custom component fv:spear_use_lv2
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:resistance_use", ResistanceComponent);
});

const StrengthComponent = {
    onCompleteUse(event) {
        const player = event.source; // Lấy người chơi (source entity)
        const itemStack = event.itemStack; // Lấy item stack hoàn thành việc sử dụng

        if (player instanceof Entity && itemStack) {
            // Áp dụng hiệu ứng strength cấp 0 trong 30 giây
            player.addEffect("strength", 10 * 20, {
                amplifier: 1,  // Cấp độ 0
                showParticles: true
            });

            console.log("Applied speed to player.");
        } else {
            console.warn("Source is not a valid player or itemStack is missing.");
        }
    }
};

// Đăng ký custom component fv:spear_use_lv2
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:strength_use", StrengthComponent);
});

const SoundTotemComponent = {
    onCompleteUse(event) {
        const player = event.source; // Lấy người chơi (source entity)
        const itemStack = event.itemStack; // Lấy item stack hoàn thành việc sử dụng

        if (player instanceof Entity && itemStack) {
            // Lấy vị trí của người chơi
            const position = player.location;

            // Phát âm thanh totem tại vị trí người chơi
            const dimension = player.dimension;
            dimension.runCommandAsync(`playsound random.totem @a ${position.x} ${position.y} ${position.z} 2`).then(() => {
                console.log("Played totem sound at player's position.");
            }).catch(error => {
                console.error("Failed to play totem sound:", error);
            });
        } else {
            console.warn("Source is not a valid player or itemStack is missing.");
        }
    }
};

// Đăng ký custom component fv:totem_sound_use
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:totem_sound_use", SoundTotemComponent);
});

const SoundAnvilBreakComponent = {
    onCompleteUse(event) {
        const player = event.source; // Lấy người chơi (source entity)
        const itemStack = event.itemStack; // Lấy item stack hoàn thành việc sử dụng

        if (player instanceof Entity && itemStack) {
            // Lấy vị trí của người chơi
            const position = player.location;

            // Phát âm thanh totem tại vị trí người chơi
            const dimension = player.dimension;
            dimension.runCommandAsync(`playsound random.anvil_break @a ${position.x} ${position.y} ${position.z} 2`).then(() => {
                console.log("Played totem sound at player's position.");
            }).catch(error => {
                console.error("Failed to play totem sound:", error);
            });
        } else {
            console.warn("Source is not a valid player or itemStack is missing.");
        }
    }
};

// Đăng ký custom component fv:totem_sound_use
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:anvil_break_sound", SoundAnvilBreakComponent);
});