import { world, EquipmentSlot, GameMode, Player, system, Entity } from "@minecraft/server";

const DecrementStack = {
    onUse({ source }) {
        if (!source) return;

        const equippable = source.getComponent("minecraft:equippable");
        if (!equippable) return;

        const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
        if (!mainhand.hasItem()) return;

        if (source.getGameMode() !== GameMode.creative) {
            if (mainhand.amount > 1) {
                mainhand.amount--; // Remove one item from the stack
            } else {
                mainhand.setItem(undefined); // Remove the item stack
            }
        }
    }
}

// Đăng ký custom component fv:anvil_break_sound
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:decrement_stack", DecrementStack);
});

const DurabilityOnCompleteUseComponent = {
    onCompleteUse(event) {
        const player = event.source;

        // Kiểm tra nếu nguồn là một Player
        if (!(player instanceof Player)) return;

        const equippable = player.getComponent("minecraft:equippable");
        if (!equippable) return;

        const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
        if (!mainhand.hasItem()) return;

        // Bỏ qua nếu ở chế độ sáng tạo
        if (player.getGameMode() === GameMode.creative) return;

        const itemStack = mainhand.getItem();
        const durability = itemStack.getComponent("minecraft:durability");
        if (!durability) return;

        // Xác định mức độ của phù phép "unbreaking"
        const enchantable = itemStack.getComponent("minecraft:enchantable");
        const unbreakingLevel = enchantable?.getEnchantment("unbreaking")?.level || 0;
        const damageChance = durability.getDamageChance(unbreakingLevel) / 100;

        // Xác suất giảm độ bền dựa trên unbreaking level
        if (Math.random() > damageChance) return;

        // Tăng độ bền đã sử dụng và kiểm tra nếu vật phẩm bị phá hủy
        const shouldBreak = durability.damage === durability.maxDurability - 1;

        if (shouldBreak) {
            mainhand.setItem(undefined); // Loại bỏ vật phẩm khi hết độ bền
            player.playSound("random.break"); // Phát âm thanh khi vật phẩm bị phá hủy
        } else {
            durability.damage++; // Tăng độ bền bị sử dụng
            mainhand.setItem(itemStack); // Cập nhật lại vật phẩm vào tay
        }
    }
};

// Đăng ký custom component fv:durability_on_use
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:durability_on_complete_use", DurabilityOnCompleteUseComponent);
});

const DurabilityOnUseComponent = {
    onUse(event) {
        const player = event.source;

        // Kiểm tra nếu nguồn là một Player
        if (!(player instanceof Player)) return;

        const equippable = player.getComponent("minecraft:equippable");
        if (!equippable) return;

        const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
        if (!mainhand.hasItem()) return;

        // Bỏ qua nếu ở chế độ sáng tạo
        if (player.getGameMode() === GameMode.creative) return;

        const itemStack = mainhand.getItem();
        const durability = itemStack.getComponent("minecraft:durability");
        if (!durability) return;

        // Xác định mức độ của phù phép "unbreaking"
        const enchantable = itemStack.getComponent("minecraft:enchantable");
        const unbreakingLevel = enchantable?.getEnchantment("unbreaking")?.level || 0;
        const damageChance = durability.getDamageChance(unbreakingLevel) / 100;

        // Xác suất giảm độ bền dựa trên unbreaking level
        if (Math.random() > damageChance) return;

        // Tăng độ bền đã sử dụng và kiểm tra nếu vật phẩm bị phá hủy
        const shouldBreak = durability.damage === durability.maxDurability - 1;

        if (shouldBreak) {
            mainhand.setItem(undefined); // Loại bỏ vật phẩm khi hết độ bền
            player.playSound("random.break"); // Phát âm thanh khi vật phẩm bị phá hủy
        } else {
            durability.damage++; // Tăng độ bền bị sử dụng
            mainhand.setItem(itemStack); // Cập nhật lại vật phẩm vào tay
        }
    }
};

// Đăng ký custom component fv:durability_on_use
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:durability_on_use", DurabilityOnUseComponent);
});

const DurabilityHitBlockComponent = {
    onMineBlock({ source }) {
        // Get main hand slot
        if (!(source instanceof Player)) return;

        const equippable = source.getComponent("minecraft:equippable");
        if (!equippable) return;

        const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
        if (!mainhand.hasItem()) return;

        // Apply durability damage when not in creative mode
        if (source.getGameMode() === GameMode.creative) return;

        const itemStack = mainhand.getItem(); // Allows us to get item components

        const durability = itemStack.getComponent("minecraft:durability");
        if (!durability) return;

        // Factor in unbreaking enchantment
        const enchantable = itemStack.getComponent("minecraft:enchantable");
        const unbreakingLevel = enchantable?.getEnchantment("unbreaking")?.level;

        const damageChance = durability.getDamageChance(unbreakingLevel) / 100;

        if (Math.random() > damageChance) return; // Randomly skip damage based on unbreaking level

        // Damage the item
        const shouldBreak = durability.damage === durability.maxDurability;

        if (shouldBreak) {
            mainhand.setItem(undefined); // Remove the item
            source.playSound("random.break"); // Play break sound
        } else {
            durability.damage++; // Increase durability damage
            mainhand.setItem(itemStack); // Update item in main hand
        }
    }
};

world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:durability_hit_block", DurabilityHitBlockComponent);
});

const CaptainScrollComponent = {
    onUse(event) {
        const player = event.source;
        const item = event.itemStack;

        // Lấy danh sách thực thể trong khoảng cách 4 blocks
        const entities = player.dimension.getEntities({
            location: player.location,
            maxDistance: 4
        });

        // Lọc thực thể thuộc type family 'becomespecial' và 'swordman'
        const targetEntity = entities.find(entity =>
            entity.hasComponent("minecraft:type_family") &&
            entity.getComponent("minecraft:type_family").hasTypeFamily("irongolem") &&
            entity.getComponent("minecraft:type_family").hasTypeFamily("swordman")
        );

        if (targetEntity) {
            // Thực hiện sự kiện 'become_cavalry' cho thực thể
            try {
                targetEntity.triggerEvent("become_captain");
            } catch (error) {
                player.runCommandAsync("say Event không kích hoạt được.");
            }

            // Phát âm thanh beacon
            player.runCommandAsync("playsound random.levelup @a ~ ~ ~ 2");

            // Sau khi điều kiện thỏa mãn, giảm số lượng item hoặc xóa stack
            const equippable = player.getComponent("minecraft:equippable");
            if (equippable) {
                const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
                if (mainhand.hasItem() && player.getGameMode() !== GameMode.creative) {
                    if (mainhand.amount > 1) {
                        mainhand.amount--; // Remove one item from the stack
                    } else {
                        mainhand.setItem(undefined); // Remove the item stack
                    }
                }
            }
        } else {
            // Hiển thị thông báo khi không có thực thể thích hợp ở gần
            player.runCommandAsync(`title @s actionbar You need to be near an alliance swordsman or use it directly on units that can become captains in other factions.`);
        }
    }
};

// Đăng ký custom component 'fv:captain_scroll'
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:captain_scroll", CaptainScrollComponent);
});

const LevelScrollComponent = {
    onUse(event) {
        const player = event.source;
        const item = event.itemStack;

        // Lấy danh sách thực thể trong khoảng cách 4 blocks
        const entities = player.dimension.getEntities({
            location: player.location,
            maxDistance: 4
        });

        // Lọc thực thể thuộc type family 'becomespecial' và 'swordman'
        const targetEntity = entities.find(entity =>
            entity.hasComponent("minecraft:type_family") &&
            entity.getComponent("minecraft:type_family").hasTypeFamily("irongolem") &&
            entity.getComponent("minecraft:type_family").hasTypeFamily("canlevelup")
        );

        if (targetEntity) {
            // Thực hiện sự kiện 'become_cavalry' cho thực thể
            try {
                targetEntity.triggerEvent("level_up");
            } catch (error) {
                player.runCommandAsync("say Event không kích hoạt được.");
            }

            // Phát âm thanh beacon
            player.runCommandAsync("playsound random.levelup @a ~ ~ ~ 2");

            // Sau khi điều kiện thỏa mãn, giảm số lượng item hoặc xóa stack
            const equippable = player.getComponent("minecraft:equippable");
            if (equippable) {
                const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
                if (mainhand.hasItem() && player.getGameMode() !== GameMode.creative) {
                    if (mainhand.amount > 1) {
                        mainhand.amount--; // Remove one item from the stack
                    } else {
                        mainhand.setItem(undefined); // Remove the item stack
                    }
                }
            }
        } else {
            // Hiển thị thông báo khi không có thực thể thích hợp ở gần
            player.runCommandAsync(`title @s actionbar You need to be near a soldier can level up`);
        }
    }
};

// Đăng ký custom component 'fv:captain_scroll'
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:level_scroll", LevelScrollComponent);
});

const BookOfClericComponent = {
    onUse(event) {
        const player = event.source; // Lấy người chơi (source entity)
        const itemStack = event.itemStack; // Lấy item stack hoàn thành việc sử dụng

        if (player instanceof Entity && itemStack) {
            // Áp dụng hiệu ứng absorption cấp 2 trong 30 giây
            player.addEffect("absorption", 20 * 30, {
                amplifier: 2,  // Cấp độ 2
                showParticles: true
            });

            // Áp dụng hiệu ứng resistance cấp 0 trong 20 giây
            player.addEffect("resistance", 20 * 20, {
                amplifier: 0,  // Cấp độ 0
                showParticles: true
            });

            // Áp dụng hiệu ứng regeneration cấp 1 trong 10 giây
            player.addEffect("regeneration", 20 * 10, {
                amplifier: 1,  // Cấp độ 1
                showParticles: true
            });

            console.log("Applied absorption, resistance, and regeneration to player.");
        } else {
            console.warn("Source is not a valid player or itemStack is missing.");
        }
    }
};

// Đăng ký custom component fv:book_of_cleric
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:book_of_cleric", BookOfClericComponent);
});

const CommanderTotemComponent = {
    onUse(event) {
        const player = event.source; // Lấy người chơi (source entity)
        const itemStack = event.itemStack; // Lấy item stack hoàn thành việc sử dụng

        // Kiểm tra và kích hoạt cooldown cho item
        const cooldownComponent = itemStack.getComponent("minecraft:cooldown");
        if (cooldownComponent && cooldownComponent.isValid()) {
            cooldownComponent.startCooldown(player);
        } else {
            console.warn("Cooldown component is not valid or not present on this item.");
            return; // Kết thúc nếu cooldown không hợp lệ
        }

        if (player instanceof Entity && itemStack) {
            // Áp dụng hiệu ứng absorption cấp 2 trong 30 giây
            player.addEffect("absorption", 20 * 600, {
                amplifier: 4,  // Cấp độ 2
                showParticles: true
            });

            // Áp dụng hiệu ứng resistance cấp 0 trong 20 giây
            player.addEffect("resistance", 20 * 600, {
                amplifier: 1,  // Cấp độ 0
                showParticles: true
            });

            // Áp dụng hiệu ứng regeneration cấp 1 trong 10 giây
            player.addEffect("regeneration", 20 * 600, {
                amplifier: 5,  // Cấp độ 1
                showParticles: true
            });

            player.addEffect("fire_resistance", 20 * 600, {
                amplifier: 1,  // Cấp độ 1
                showParticles: true
            });

            player.addEffect("health_boost", 20 * 600, {
                amplifier: 2,  // Cấp độ 1
                showParticles: true
            });

            console.log("Applied absorption, resistance, and regeneration to player.");
        } else {
            console.warn("Source is not a valid player or itemStack is missing.");
        }
    }
};

// Đăng ký custom component fv:book_of_cleric
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:commander_totem", CommanderTotemComponent);
});