import { world } from "@minecraft/server";

// Custom component để áp dụng sát thương khi tấn công và gọi function hammer_sound
const LegonSwordComponent = {
    onHitEntity({ hitEntity, source }) {
        let chance = 0.10;  // 10% cơ hội gây sát thương
        let extraDamage = 60;  // Sát thương thêm

        // Áp dụng sát thương nếu xác suất xảy ra
        if (Math.random() < chance) {
            hitEntity.applyDamage(extraDamage);
            console.log(`Đã gây ${extraDamage} sát thương lên thực thể.`);

            // Gọi function hammer_sound trong cả 3 dimension
            const dimensions = ["overworld", "nether", "the_end"];
            dimensions.forEach(dimensionName => {
                const dimension = world.getDimension(dimensionName);
                if (dimension) {
                    dimension.runCommandAsync('function big_sword_sound').then(() => {
                        console.log(`Executed hammer_sound in ${dimensionName}`);
                    }).catch(error => {
                        console.error(`Failed to execute in ${dimensionName}:`, error);
                    });
                }
            });
        }
    }
};

// Đăng ký custom component fv:big_sword
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:legon_sword", LegonSwordComponent);
});

const LegonSwordEffectComponent = {
    onHitEntity({ hitEntity }) {
        let chance = 0.50;  // Mặc định là 10% cơ hội
        let effectDuration = 15;  // Mặc định là 5 giây
        let effectAmplifier = 2;  // Cấp độ của hiệu ứng luôn là 10

        // Áp dụng hiệu ứng nếu xác suất xảy ra
        if (Math.random() < chance) {
            hitEntity.addEffect("weakness", effectDuration * 20, {
                amplifier: effectAmplifier,
                showParticles: true
            });
        }
    }
};

// Đăng ký custom component fv:spear_effect
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:legon_sword_effect", LegonSwordEffectComponent);
});

const LegonSwordEffectMaxComponent = {
    onHitEntity({ hitEntity }) {
        let chance = 0.50;  // Mặc định là 10% cơ hội
        let effectDuration = 25;  // Mặc định là 5 giây
        let effectAmplifier = 2;  // Cấp độ của hiệu ứng luôn là 10

        // Áp dụng hiệu ứng nếu xác suất xảy ra
        if (Math.random() < chance) {
            hitEntity.addEffect("weakness", effectDuration * 20, {
                amplifier: effectAmplifier,
                showParticles: true
            });
        }
    }
};

// Đăng ký custom component fv:spear_effect
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:legon_sword_effect_max", LegonSwordEffectMaxComponent);
});