import { world, Entity } from '@minecraft/server';

const SteelBloodVampiricCleaverEffectComponent = {
    onHitEntity({ hitEntity }) {
        let chance = 0.20;  // Mặc định là 10% cơ hội
        let effectDuration = 10;  // Mặc định là 5 giây
        let effectAmplifier = 1;  // Cấp độ của hiệu ứng luôn là 10

        // Áp dụng hiệu ứng nếu xác suất xảy ra
        if (Math.random() < chance) {
            hitEntity.addEffect("weakness", effectDuration * 20, {
                amplifier: effectAmplifier,
                showParticles: true
            });
        }
    }
};

// Đăng ký custom component fv:spear_effect
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:steelblood_vampiric_cleaver_weakness", SteelBloodVampiricCleaverEffectComponent);
});

const SteelBloodVampiricCleaverHungerComponent = {
    onHitEntity({ hitEntity }) {
        let chance = 0.20;  // Mặc định là 10% cơ hội
        let effectDuration = 20;  // Mặc định là 5 giây
        let effectAmplifier = 1;  // Cấp độ của hiệu ứng luôn là 10

        // Áp dụng hiệu ứng nếu xác suất xảy ra
        if (Math.random() < chance) {
            hitEntity.addEffect("hunger", effectDuration * 20, {
                amplifier: effectAmplifier,
                showParticles: true
            });
        }
    }
};

// Đăng ký custom component fv:spear_effect
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:steelblood_vampiric_cleaver_hunger", SteelBloodVampiricCleaverHungerComponent);
});

const SteelBloodVampiricCleaverHitComponent = {
    onHitEntity({ hitEntity }) {
        let chance = 0.10;  // Mặc định là 3% cơ hội
        let effectDuration = 1.5;  // Mặc định là 1 giây
        let effectAmplifier = 0;  // Cấp độ của hiệu ứng luôn là 1

        // Áp dụng hiệu ứng nếu xác suất xảy ra
        if (Math.random() < chance) {
            hitEntity.addEffect("instant_damage", effectDuration * 20, {
                amplifier: effectAmplifier,
                showParticles: true
            });

            // Lấy vị trí của thực thể bị đánh trúng
            const position = hitEntity.location;
            const dimension = hitEntity.dimension;

            // Phát âm thanh tại vị trí của thực thể bị đánh trúng
            dimension.runCommandAsync(`playsound hit.bonus @a ${position.x} ${position.y} ${position.z} 4`).then(() => {
                console.log(`Executed hammer_sound at position ${position.x}, ${position.y}, ${position.z}`);
            }).catch(error => {
                console.error(`Failed to execute playsound at position ${position.x}, ${position.y}, ${position.z}:`, error);
            });
        }
    }
};

// Đăng ký custom component fv:battle_axe
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:steelblood_vampiric_cleaver_hit", SteelBloodVampiricCleaverHitComponent);
});

// Định nghĩa component fv:vampiric_health
const VampiricHealthComponent = {
    onHitEntity({ hitEntity }) {
        // Xác suất 20% kích hoạt hiệu ứng
        if (Math.random() < 0.2) {
            // Lấy vị trí của thực thể bị đánh trúng
            const position = hitEntity.location;
            const dimension = hitEntity.dimension;

            // Áp dụng hiệu ứng regeneration cho người chơi cầm vũ khí
            dimension.runCommandAsync(`effect @p[c=1,hasitem={item=fv:steelblood_vampiric_cleaver,location=slot.weapon.mainhand}] regeneration 10 0`).then(() => {
                console.log(`Applied regeneration effect to the player with steelblood vampiric cleaver at position ${position.x}, ${position.y}, ${position.z}`);
            }).catch(error => {
                console.error(`Failed to apply regeneration effect at position ${position.x}, ${position.y}, ${position.z}:`, error);
            });
        }
    }
};

// Đăng ký component fv:vampiric_health trong hệ thống
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:vampiric_health", VampiricHealthComponent);
});

// Định nghĩa component fv:vampiric_health
const VampiricHealthComponentV2 = {
    onHitEntity({ hitEntity }) {
        // Xác suất 20% kích hoạt hiệu ứng
        if (Math.random() < 0.2) {
            // Lấy vị trí của thực thể bị đánh trúng
            const position = hitEntity.location;
            const dimension = hitEntity.dimension;

            // Áp dụng hiệu ứng regeneration cho người chơi cầm vũ khí
            dimension.runCommandAsync(`effect @p[c=1,hasitem={item=fv:steelblood_vampiric_cleaver_v2,location=slot.weapon.mainhand}] regeneration 10 1`).then(() => {
                console.log(`Applied regeneration effect to the player with steelblood vampiric cleaver at position ${position.x}, ${position.y}, ${position.z}`);
            }).catch(error => {
                console.error(`Failed to apply regeneration effect at position ${position.x}, ${position.y}, ${position.z}:`, error);
            });
        }
    }
};

// Đăng ký component fv:vampiric_health trong hệ thống
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:vampiric_health_v2", VampiricHealthComponentV2);
});

// Định nghĩa component fv:vampiric_health
const VampiricHealthComponentV3 = {
    onHitEntity({ hitEntity }) {
        // Xác suất 20% kích hoạt hiệu ứng
        if (Math.random() < 0.2) {
            // Lấy vị trí của thực thể bị đánh trúng
            const position = hitEntity.location;
            const dimension = hitEntity.dimension;

            // Áp dụng hiệu ứng regeneration cho người chơi cầm vũ khí
            dimension.runCommandAsync(`effect @p[c=1,hasitem={item=fv:steelblood_vampiric_cleaver_v3,location=slot.weapon.mainhand}] regeneration 10 2`).then(() => {
                console.log(`Applied regeneration effect to the player with steelblood vampiric cleaver at position ${position.x}, ${position.y}, ${position.z}`);
            }).catch(error => {
                console.error(`Failed to apply regeneration effect at position ${position.x}, ${position.y}, ${position.z}:`, error);
            });
        }
    }
};

// Đăng ký component fv:vampiric_health trong hệ thống
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fv:vampiric_health_v3", VampiricHealthComponentV3);
});